@@ -1,7 +1,6 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2006-2007 Imendio AB
- * Copyright (C) 2007 Collabora Ltd.
+ * Copyright (C) 2007-2008 Collabora Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -15,8 +14,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  *
  * Authors: Martyn Russell <martyn@imendio.com>
  *          Xavier Claessens <xclaesse@gmail.com>
@@ -26,19 +25,20 @@
 #define __EMPATHY_GEOMETRY_H__
 
 #include <glib.h>
+#include <gtk/gtk.h>
 
 G_BEGIN_DECLS
 
-void empathy_geometry_save (const gchar *name,
-			   gint         x,
-			   gint         y,
-			   gint         w,
-			   gint         h);
-void empathy_geometry_load (const gchar *name,
-			   gint        *x,
-			   gint        *y,
-			   gint        *w,
-			   gint        *h);
+void empathy_geometry_save (GtkWindow *window,
+    const gchar *name);
+
+void empathy_geometry_load (GtkWindow *window,
+    const gchar *name);
+
+void empathy_geometry_bind (GtkWindow *window,
+    const gchar *name);
+
+void empathy_geometry_unbind (GtkWindow *window);
 
 G_END_DECLS
 
