@@ -334,12 +334,17 @@ import_widget_set_up_account_list (EmpathyImportWidget *self)
 }
 
 static void
-import_widget_cms_ready_cb (EmpathyConnectionManagers *cms,
-    GParamSpec *pspec,
-    EmpathyImportWidget *self)
+import_widget_cms_prepare_cb (GObject *source,
+    GAsyncResult *result,
+    gpointer user_data)
 {
-  if (empathy_connection_managers_is_ready (cms))
-    import_widget_set_up_account_list (self);
+  EmpathyImportWidget *self = user_data;
+
+  if (!empathy_connection_managers_prepare_finish (
+        EMPATHY_CONNECTION_MANAGERS (source), result, NULL))
+    return;
+
+  import_widget_set_up_account_list (self);
 }
 
 static void
@@ -438,11 +443,8 @@ do_constructed (GObject *obj)
   g_signal_connect (priv->vbox, "destroy",
       G_CALLBACK (import_widget_destroy_cb), self);
 
-  if (empathy_connection_managers_is_ready (priv->cms))
-    import_widget_set_up_account_list (self);
-  else
-    g_signal_connect (priv->cms, "notify::ready",
-        G_CALLBACK (import_widget_cms_ready_cb), self);
+  empathy_connection_managers_prepare_async (priv->cms,
+      import_widget_cms_prepare_cb, self);
 }
 
 static void
